package com.jenny.magic.items;

import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public class TeleportScroll extends BaseItem {
    public TeleportScroll(Properties pProperties) {
        super(pProperties);
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer.position(), pPlayer, pPlayer.getItemInHand(pUsedHand), pLevel);
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getClickedPos().getCenter().add(0, 0.5, 0), pContext.getPlayer(), pContext.getItemInHand(), pContext.getLevel());

        return InteractionResult.SUCCESS;
    }

    protected void use(Vec3 pos, Player player, @NotNull ItemStack itemStack, Level level) {
        if (itemStack.getTag() == null) {
            itemStack.setTag(new CompoundTag());
        }
        if (locationSet(itemStack.getTag()) && !player.isCrouching()) {
            if (level.isClientSide) {
                spawnParticles(level, player.position().add(0, player.getEyeHeight(), 0), 20, 5, ParticleTypes.POOF, ParticleDirection.INWARD);
            }
            player.setPos(getLocation(itemStack.getTag()));
            if (level.isClientSide) {
                spawnParticles(level, player.position().add(0, player.getEyeHeight(), 0), 20, 0.2, ParticleTypes.END_ROD, ParticleDirection.OUTWARD);
            }
        } else {
            itemStack.setTag(setLocation(pos, itemStack.getTag()));
            if (level.isClientSide) {
                MessageLocationSet();
            }
        }
    }

    private boolean locationSet(@NotNull CompoundTag cTag) {
        return cTag.contains("teleport_x") && cTag.contains("teleport_y") && cTag.contains("teleport_z");
    }

    private CompoundTag setLocation(@NotNull Vec3 playerPos, @NotNull CompoundTag cTag) {
        cTag.putDouble("teleport_x", playerPos.x);
        cTag.putDouble("teleport_y", playerPos.y);
        cTag.putDouble("teleport_z", playerPos.z);
        return cTag;
    }

    private Vec3 getLocation(@NotNull CompoundTag cTag) {
        return new Vec3(
                cTag.getDouble("teleport_x"),
                cTag.getDouble("teleport_y"),
                cTag.getDouble("teleport_z")
        );
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        if (pStack.getTag() != null && locationSet(pStack.getTag())) {
            String key = String.format("tooltip.%s.scroll_teleport.set", MODID);
            MutableComponent toolTip = Component.translatable(key);
            Vec3 pos = getLocation(pStack.getTag());
            MutableComponent toolTipPos = Component.literal((int) pos.x + ";" + (int) pos.y + ";" + (int) pos.z);
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE).append(toolTipPos.withStyle(ChatFormatting.WHITE)));
        } else {
            String key = String.format("tooltip.%s.scroll_teleport.unset", MODID);
            MutableComponent toolTip = Component.translatable(key);
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE));
        }

    }

    protected void MessageLocationSet() {
        Minecraft.getInstance().player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport.set", MODID)));
    }
}
